/**
 * \file   l4re/lib/src/fb.cc
 * \brief  Framebuffer
 */
/*
 * (c) 2008-2009 Technische Universität Dresden
 * This file is part of TUD:OS and distributed under the terms of the
 * GNU General Public License 2.
 * Please see the COPYING-GPL-2 file for details.
 *
 * As a special exception, you may use this file as part of a free software
 * library without restriction.  Specifically, if other files instantiate
 * templates or use macros or inline functions from this file, or you compile
 * this file and link it with other files to produce an executable, this
 * file does not by itself cause the resulting executable to be covered by
 * the GNU General Public License.  This exception does not however
 * invalidate any other reasons why the executable file might be covered by
 * the GNU General Public License.
 */
#include <l4/re/fb>
#include <l4/re/fb-sys.h>
#include <l4/re/dataspace>
#include <l4/re/protocols>

#include <l4/cxx/exceptions>
#include <l4/cxx/ipc_helper>
#include <l4/cxx/ipc_stream>

#include <l4/sys/err.h>

inline
L4::Ipc_istream &operator >> (L4::Ipc_istream &s, L4Re::Framebuffer::Info &vi)
{ s.get(vi); return s; }

namespace L4Re
{

using L4::Opcode;

int
Framebuffer::info(Info *info) const throw()
{
  L4::Ipc_iostream io(l4_utcb());
  io << Opcode(Framebuffer_::Info);
  long err = l4_error(io.call(cap(), L4Re::Protocol::Fb));
  if (EXPECT_FALSE(err < 0))
    return err;

  io >> *info;
  return err;
}

int
Framebuffer::mem(L4::Cap<Dataspace> ds) const throw()
{
  L4::Ipc_iostream io(l4_utcb());
  io << Opcode(Framebuffer_::Get_fb);
  io << L4::Small_buf(ds.cap());
  return l4_error(io.call(cap(), L4Re::Protocol::Fb));
}

int
Framebuffer::refresh(int x, int y, int w, int h) const throw()
{
  L4::Ipc_iostream io(l4_utcb());
  io << Opcode(Framebuffer_::Refresh);
  io << x << y << w << h;
  return l4_error(io.call(cap(), L4Re::Protocol::Fb));
}

}
