/**
 * \file
 * \brief Service C interface.
 */
/*
 * (c) 2008-2009 Technische Universität Dresden
 * This file is part of TUD:OS and distributed under the terms of the
 * GNU General Public License 2.
 * Please see the COPYING-GPL-2 file for details.
 *
 * As a special exception, you may use this file as part of a free software
 * library without restriction.  Specifically, if other files instantiate
 * templates or use macros or inline functions from this file, or you compile
 * this file and link it with other files to produce an executable, this
 * file does not by itself cause the resulting executable to be covered by
 * the GNU General Public License.  This exception does not however
 * invalidate any other reasons why the executable file might be covered by
 * the GNU General Public License.
 */
#pragma once

/**
 * \defgroup api_l4re_c_service Service interface
 * \ingroup api_l4re_c
 * \brief Service C interface.
 */

#include <l4/sys/types.h>

EXTERN_C_BEGIN

/**
 * \brief Service type.
 */
typedef l4_cap_idx_t l4re_srvc_t;

/**
 * \ingroup api_l4re_c_service
 * \brief Open a service.
 *
 * \param server     Server to talk to.
 * \param servicecap Capability to put service into.
 * \param data       Data
 * \param len        Length of data in bytes.
 *
 * \return 0 for success, <0 on error
 *
 * \see L4Re::Service::open
 */
L4_CV long
l4re_srvc_open(const l4re_srvc_t server,
               const l4_cap_idx_t servicecap,
               const char *data, unsigned long len) L4_NOTHROW;

EXTERN_C_END
