/**
 * \file
 * \brief Framebuffer utility functionality.
 */
/*
 * (c) 2009 Technische Universität Dresden
 * This file is part of TUD:OS and distributed under the terms of the
 * GNU General Public License 2.
 * Please see the COPYING-GPL-2 file for details.
 *
 * As a special exception, you may use this file as part of a free software
 * library without restriction.  Specifically, if other files instantiate
 * templates or use macros or inline functions from this file, or you compile
 * this file and link it with other files to produce an executable, this
 * file does not by itself cause the resulting executable to be covered by
 * the GNU General Public License.  This exception does not however
 * invalidate any other reasons why the executable file might be covered by
 * the GNU General Public License.
 */
#pragma once

#include <l4/sys/compiler.h>
#include <l4/re/c/fb.h>
#include <l4/sys/err.h>

EXTERN_C_BEGIN

/**
 * \brief Query the framebuffer and attach to it.
 * \ingroup api_l4re_c_util
 * \retval fb_cap   Framebuffer capability.
 * \retval fbds     Framebuffer memory capability.
 * \retval fb_addr  Virtual address of the mapped framebuffer.
 *
 * For further explanations please refer to L4Re::Util::Fb::get
 */
L4_CV L4_INLINE int
l4re_util_fb_get(l4re_fb_t *fb_cap, l4re_ds_t *fbds, void **fb_addr);



/* Implementations */

#include <l4/re/c/util/cap_alloc.h>
#include <l4/re/c/namespace.h>
#include <l4/re/c/rm.h>
#include <l4/re/c/fb.h>

L4_CV L4_INLINE int
l4re_util_fb_get(l4re_fb_t *fb_cap, l4re_ds_t *fbds, void **fb_addr)
{
  int err;
  const char *const fb_ns_name = "fb";

  if (l4_is_invalid_cap(*fb_cap = l4re_util_cap_alloc()))
    return -L4_ENOMEM;

  if ((err = l4re_ns_query(fb_ns_name, *fb_cap)))
    return err;

  if (l4_is_invalid_cap(*fbds = l4re_util_cap_alloc()))
    return -L4_ENOMEM;

  if ((err = l4re_fb_mem(*fb_cap, *fbds)))
    return err;

  *fb_addr = 0;
  if ((err = l4re_rm_attach(fb_addr, l4re_ds_size(*fbds),
                            L4RE_RM_SEARCH_ADDR, *fbds,
                            0, L4_SUPERPAGESHIFT)))
    return err;

  return 0;
}

EXTERN_C_END
