/*
 * (c) 2008-2009 Technische Universität Dresden
 * This file is part of TUD:OS and distributed under the terms of the
 * GNU General Public License 2.
 * Please see the COPYING-GPL-2 file for details.
 */
#include "loader_script.h"
#include "debug.h"
#include "globals.h"
#include "name_space.h"
#include "script_invoke.h"

#include <l4/libloader/name_space_cfg>
#include <l4/sigma0/sigma0.h>
#include <l4/re/service>


static Dbg info(Dbg::Name_space, "ns");
static Moe::Name_space ns_ns;

using Ldr::Names_parser;
using Ldr::Parse_exception;

namespace Moe {
namespace Names {
  using namespace L4Re::Util::Names;
}
using Moe::Name_space;


class Ldr_names_obj_mgr : public Moe_obj_mgr
{
public:
  Ldr::Parser::Val find_object(cxx::String const &name,
                               Ldr::Scanner::Context const &ctxt) const
  {
    // look in a local name space
    Names::Entry *src = root_name_space()->find_iter(name);
    if (!src)
      throw Ldr::Parse_exception(ctxt, Ldr::Token(Ldr::Token::Name, name),
	  "object not found:");
    return src;
  }

};

class Ldr_names_parser : public Names_parser
{
private:
  Ldr_names_obj_mgr mgr;
public:
  Ldr_names_parser(Ldr::Scanner *scan, Name_space *ns)
  : Names_parser(scan, ns, &mgr) {}

  void check_invocation(Val const &)
  {
    return;
    throw Parse_exception(_scan, Ldr::Token(),
                          "invocation of object not supported");
  }

  Val find_object(Ldr::Token const &name)
  {
    // look in a local name space
    Names::Entry *src = root_name_space()->find_iter(name.text());
    if (!src)
      throw Parse_exception(_scan, name, "object not found:");
    return src;
  }

  void entry();

  Val invoke(Val const &obj, Ldr::Token const &func);
};

void
Ldr_names_parser::entry()
{
//  Names::Name new_name = namedup(cur().text());
  Entry *e = new Entry(cur().text(), Names::Obj());
  int err = _ns->insert(e);
  if (err)
    {
//      freename(new_name);
      delete e;
      throw Parse_exception(_scan, cur(), "name already exists:");
    }
  cap_rval(e);
}


void
read_namespaces(Moe::Loader_script *cfg)
{
  L4::Cap<void> ns_ns_cap = object_pool.cap_alloc()->alloc(&ns_ns);
  root_name_space()->register_obj("ns", Names::Obj(Names::Obj::F_dir, &ns_ns));
  cfg->rewind_stream();
  cxx::String name;
  while (cfg->goto_section("namespace", &name))
    {
      if (name.empty())
	{
	  Err().printf("%s:%d: cannot create anonymous name space, skip\n",
	               cfg->file(), cfg->line() - 1);
	  continue;
	}

      Names::Entry *n = ns_ns.find(name);

      if (n)
	{
	  Err().printf("%s:%d: object with name '%.*s' already exists, skip.\n",
	               cfg->file(), cfg->line() - 1, name.len(), name.start());
	  continue;
	}

      Name_space *ns = new Name_space();
      if (!ns)
	{
	  Err().printf("%s:%d: cannot allocate name space, out of memory\n",
	               cfg->file(), cfg->line() - 1);
	  continue;
	}

      enum { Ns_flags = Names::Obj::F_dir | Names::Obj::F_rw };
      L4::Cap<void> ns_cap = object_pool.cap_alloc()->alloc(ns);

      info.printf("added name space '%.*s'\n", name.len(), name.start());

      Ldr::Scanner s2 = cfg->block(1);
      Ldr_names_parser nparse(&s2, ns);
      nparse.parse(Err());
      if (nparse.errors())
	{
	  Err().printf("=================================================\n");
	  Err().printf("could not create name space: %.*s\n", name.len(), name.start());
	  Err().printf("=================================================\n");
	  // XXX: fix memory leak we should free the name space (ns)
	  continue;
	}
      ns_ns.register_obj(name, Names::Obj(Ns_flags, ns));

      if (info.is_active())
	{
	  info.printf("dump of user-define name space: '%.*s'\n",
	              name.len(), name.start());
	  ns->dump();
	}
    }

  if (Dbg(Dbg::Boot).is_active())
    {
      info.printf("dump of user defined name spaces\n");
      ns_ns.dump(true);
    }

}

}
